<?php
require ("common.php");
?>
<html>
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
	<link type="text/css" rel="stylesheet" href="styles.css"/>
</head>
<body onload="run ()">

<!-- Title (start) -->
<div class="titleBar">
<div class="titleTitle">Déboguer facilement un service Web en JavaScript et PHP (v1)</div>
<div class="titleCopyright">Cette oeuvre est mise à disposition selon les termes de la <a href="http://creativecommons.org/licenses/by-nc/4.0/" target="_blank">Licence Creative Commons Attribution - Pas d’Utilisation Commerciale 4.0 International</a>.
</div>
<div class="titleDescription">
<div class="paragraph">
Réalisé par Denis Duplan pour <a href="http://www.stashofcode.fr" target="_blank">Stash of Code</a> (<a href="mailto:stashofcode@gmail.com">stashofcode@gmail.com</a>) en mars 2019.
</div>
<div class="paragraph">
Dans cet exemple, il s'agit d'appeler plusieurs services de manière asynchrone, pour lister les bases de données, puis les tables dans une de ces bases, puis les champs dans l'une de ces tables, les uns après les autres grâce à un chaînage de promises.
</div>
</div>
</div>
<!-- Title (end) -->

<script>
//------------------------------------------------------------------------------
// Request
//------------------------------------------------------------------------------

function Request (service, params, debug=g.debug) {
	this.service = service;
	this.params = params;
	this.debug = debug;
}

//------------------------------------------------------------------------------
// Response
//------------------------------------------------------------------------------

function Response (service, code, data, debug=g.debug) {
	this.service = service;
	this.code = code;
	this.data = data;
	this.debug = debug;
}

Response.prototype.fromJSON = function (jsonResponse) {
	var response;

	response = JSON.parse (jsonResponse);
	return (new Response (response.service, response.code, response.data, response.debug));
};

//------------------------------------------------------------------------------
// Service
//------------------------------------------------------------------------------

function Service (script, service, debug=g.debug) {
	this.request = null;
	this.script = script;
	this.service = service;
	this.debug = debug;
}

Service.prototype.RESPONSE_SUCCESS = 0;
Service.prototype.RESPONSE_ERROR_PHP = 1;
Service.prototype.RESPONSE_ERROR_SERVICE = 2;

Service.prototype.log = function (message) {
	console.log (`// Log (start)\nScript:\t\t${this.script}\nService:\t${this.service}\nMessage:\t${message}\n//Log (end)`);
};

Service.prototype.run = function (params=null) {
	return (new Promise ((resolve, reject) => {
		var formData;

		this.request = new XMLHttpRequest ();
		this.request.open ("POST", this.script, true);
		this.request.onreadystatechange = ((o, resolve, reject) => {
			return (() =>  o.callback.call (o, resolve, reject));
		}) (this, resolve, reject);
		request = new Request (this.service, params, this.debug);
		formData = new FormData ();
		formData.append ("request", JSON.stringify (request));
		if (this.debug)
			this.log (`Sending\n\t\tRequest:\t${JSON.stringify (request)}\n\t\t\t\tService:\t${this.service}\n\t\t\t\tParams:\t\t${JSON.stringify (params)}\n\t\t\t\tDebug:\t\t${this.debug}`);
		this.request.send (formData);
	}));
};

Service.prototype.callback = function (resolve, reject) {
	if (this.request.readyState !== 4)
		return;
	if (this.request.status !== 200)
		this.onFailure (resolve, reject);
	else
		this.onSuccess (resolve, reject);
};

Service.prototype.onSuccess = function (resolve, reject) {
	var response;

	response = Response.prototype.fromJSON (this.request.responseText);
	switch (response.code) {
		case Service.prototype.RESPONSE_ERROR_PHP:
			if (this.debug)
				this.log (`Receiving\n\t\tError:\t\tERROR (PHP)\n\t\tFile:\t\t${response.data.file}\n\t\tLine:\t\t${response.data.line}\n\t\tMessage:\t${response.data.message}`);
			reject (response.data);
			break;
		case Service.prototype.RESPONSE_ERROR_SERVICE:
			if (this.debug)
				this.log (`Receiving\n\t\tError:\t\tERROR (SERVICE)\n\t\tFile:\t\t${response.data.file}\n\t\tLine:\t\t${response.data.line}\n\t\tMessage:\t${response.data.message}`);
			reject (response.data);
			break;
		case Service.prototype.RESPONSE_SUCCESS:
			if (this.debug)
				this.log (`Receiving\n\t\tSuccess\n\t\tResponse:\t${this.request.responseText}\n\t\t\t\tCode:\t${response.code}\n\t\t\t\tData:\t${JSON.stringify(response.data)}\n\t\t\t\tDebug:\t${response.debug}`);
			resolve (response.data);
			break;
	}
};

Service.prototype.onFailure = function (resolve, reject) {
	if (this.debug)
		this.log (`HTTP failure code ${this.request.status}`);
	reject (`HTTP failure code ${this.request.status}`);
};

//------------------------------------------------------------------------------
// Lister les bases de données
//------------------------------------------------------------------------------

function ServiceGetDatabases (debug=g.debug) {
	Service.call (this, "<?php echo (htmlspecialchars (SITE_VPATH)) ?>services.php", "GetDatabases", debug);
}
ServiceGetDatabases.prototype = Object.create (Service.prototype);

function getDatabases (debug=g.debug) {
	var service;

	service = new ServiceGetDatabases (debug);
	return (service.run ());
}

function showDatabases () {
	return (getDatabases ().then ((result) => {
		g.databases = result;
		resetSelect ("tagDatabases", g.databases);
		return (showTables ());
	}, showError));
}

//------------------------------------------------------------------------------
// Lister les tables
//------------------------------------------------------------------------------

function ServiceGetTables (debug=g.debug) {
	Service.call (this, "<?php echo (htmlspecialchars (SITE_VPATH)) ?>services.php", "GetTables", debug);
}
ServiceGetTables.prototype = Object.create (Service.prototype);

function getTables (database, debug=g.debug) {
	var service;

	service = new ServiceGetTables (debug);
	return (service.run ({ database: database }));
}

function showTables () {
	return (getTables (g.databases[document.getElementById ("tagDatabases").selectedIndex]).then ((result) => {
		g.tables = result;
		resetSelect ("tagTables", g.tables);
		return (showFields ());
	}, showError));
}

//------------------------------------------------------------------------------
// Lister les champs
//------------------------------------------------------------------------------

function ServiceGetFields (debug=g.debug) {
	Service.call (this, "<?php echo (htmlspecialchars (SITE_VPATH)) ?>services.php", "GetFields", debug);
}
ServiceGetFields.prototype = Object.create (Service.prototype);

function getFields (database, table, debug=g.debug) {
	var service;

	service = new ServiceGetFields (debug);
	return (service.run ({ database: database, table: table }));
}

function showFields () {
	return (getFields (g.databases[document.getElementById ("tagDatabases").selectedIndex], g.tables[document.getElementById ("tagTables").selectedIndex]).then ((result) => {
		var fields;

		g.fields = result;
		fields = Array.from (g.fields, field => field.name);
		resetSelect ("tagFields", fields);
		showField ();
	}, showError));
}

function showField () {
	var field;

	field = g.fields[document.getElementById ("tagFields").selectedIndex];
	document.getElementById ("tagFieldName").innerHTML = field["name"];
	document.getElementById ("tagFieldType").innerHTML = field["type"];
	document.getElementById ("tagFieldKey").innerHTML = field["key"];
}

//------------------------------------------------------------------------------
// Divers
//------------------------------------------------------------------------------

function resetSelect (tagSELECTName, items=null) {
	var tagSELECT, item, tagOPTION;

	tagSELECT = document.getElementById (tagSELECTName);
	while (tagSELECT.options.length)
		tagSELECT.options.remove (0);
	if (!items) {
		tagSELECT.disabled = true;
		return;
	}
	for (item of items) {
		tagOPTION = document.createElement ("option");
		tagOPTION.innerHTML = item;
		tagOPTION.value = item;
		tagSELECT.options.add (tagOPTION);
	}
	tagSELECT.disabled = false;
	tagSELECT.selectedIndex = 0;
}

function showError (message) {
	document.getElementById ("tagMessage").innerHTML = `ERROR: ${message}`;
}

//------------------------------------------------------------------------------
// Programme principal
//------------------------------------------------------------------------------

var g = {
	debug: true,
	databases: null,
	tables: null,
	fields: null
};

function run () {
	showDatabases ();
}
</script>

<div style="display:grid;grid-template-columns:min-content min-content;gap:5px">
	<div>Databases:</div><div><select id="tagDatabases" disabled="true" onchange="showTables ()"></select></div>
	<div>Tables:</div><div><select id="tagTables" disabled="true" onchange="showFields ()"></select></div>
	<div>Fields:</div><div><select id="tagFields" disabled="true" onchange="showField ()"></select></div>
</div>

<div id="tagMessage"></div>

<div style="display:grid;grid-template-columns:min-content min-content;gap:5px">
	<div>Name:</div><div id="tagFieldName"></div>
	<div>Type:</div><div id="tagFieldType"></div>
	<div>Key:</div><div id="tagFieldKey"></div>
</div>

</body>
</html>