<?php
/*
Cette oeuvre est mise à disposition selon les termes de la Licence Creative Commons Attribution - Pas d’Utilisation Commerciale 4.0 International :

http://creativecommons.org/licenses/by-nc/4.0/

Réalisé par Denis Duplan (stashofcode@gmail.com) pour Stash of Code (www.stashofcode.fr) en mars 2019.
*/

require ("common.php");

//------------------------------------------------------------------------------
// Divers
//------------------------------------------------------------------------------

// Récupérer un paramètre CGI

function getCGI ($variable) {
	if (isset ($_POST[$variable]))
		return ($_POST[$variable]);
	if (isset ($_GET[$variable]))
		return ($_GET[$variable]);
	return ("");
}

// Filtrer des caractères en SQL, avec suppression éventuelle du code HTML

function sqlFilter ($request, $stripTags=false) {
	if (!$stripTags)
		return (str_replace ("'", "''", $request));
	else
		return (str_replace ("'", "''", strip_tags ($request)));
}

//------------------------------------------------------------------------------
// Services
//------------------------------------------------------------------------------

// Service

define ('RESPONSE_SUCCESS', 0);
define ('RESPONSE_ERROR_PHP', 1);
define ('RESPONSE_ERROR_SERVICE', 2);

class Request {
	public $service, $params, $debug;

	function __construct ($service, $params, $debug=false) {
		$this->service = $service;
		$this->params = $params;
		$this->debug = $debug;
	}

	public static function fromJSON ($jsonRequest) {
		$request = json_decode ($jsonRequest);
		return (new Request ($request->service, $request->params, $request->debug));
	}
}

class Response {
	public $service, $code, $data, $debug;

	function __construct ($service, $code, $data, $debug=false) {
		$this->service = $service;
		$this->code = $code;
		$this->data = $data;
		$this->debug = $debug;
	}

	function toJSON () {
		$response = array ('code' => $this->code, 'data' => $this->data, 'debug' => $this->debug);
		return (json_encode ($response));
	}
}

class Service {
	public $name, $debug;

	function __construct ($name, $debug=false) {
		$this->name = $name;
		$this->debug = $debug;
	}

	function run ($params) {
		$GLOBALS['link'] = mysqli_connect (MYSQL_HOSTNAME, MYSQL_USER, MYSQL_PASSWORD);
	}

	function reply ($response) {
		header ('Content-Type: application/json');
		echo ($response);
		exit ();
	}
}

// Lister les bases de données

class GetDatabases extends Service {
	function __construct ($debug=false) {
		parent::__construct ('GetDatabases', $debug);
	}

	function run ($params) {
		parent::run ($params);
		$result = mysqli_query ($GLOBALS['link'], 'SHOW DATABASES');
		$list = array ();
		for ($count = 0 ; $count != mysqli_num_rows ($result) ; $count++) {
			$record = mysqli_fetch_array ($result);
			array_push ($list, $record[0]);
		}
		$response = new Response ($this->name, RESPONSE_SUCCESS, $list);
		$this->reply ($response->toJSON ());
	}
}

// Lister les tables

class GetTables extends Service {
	function __construct ($debug=false) {
		parent::__construct ('GetTables', $debug);
	}

	function run ($params) {
		parent::run ($params);
		mysqli_select_db ($GLOBALS['link'], $params->database);
		$result = mysqli_query ($GLOBALS['link'], 'SHOW TABLES');
		$list = array ();
		for ($count = 0 ; $count != mysqli_num_rows ($result) ; $count++) {
			$record = mysqli_fetch_array ($result);
			array_push ($list, $record[0]);
		}
		$response = new Response ($this->name, RESPONSE_SUCCESS, $list);
		$this->reply ($response->toJSON ());
	}
}

// Lister les champs

class GetFields extends Service {
	function __construct ($debug=false) {
		parent::__construct ('GetFields', $debug);
	}

	function run ($params) {
		parent::run ($params);
		mysqli_select_db ($GLOBALS['link'], $params->database);
		$result = mysqli_query ($GLOBALS['link'], 'SHOW COLUMNS FROM '.sqlFilter ($params->table));
		$list = array ();
		for ($count = 0 ; $count != mysqli_num_rows ($result) ; $count++) {
			$record = mysqli_fetch_array ($result);
			$field = [ 'name' => $record['Field'], 'type' => $record['Type'], 'key' => $record['Key'] ];
			array_push ($list, $field);
		}
		$response = new Response ($this->name, RESPONSE_SUCCESS, $list);
		$this->reply ($response->toJSON ());
	}
}

//------------------------------------------------------------------------------
// Programme principal
//------------------------------------------------------------------------------

// Prendre le contrôle de la gestion des erreurs

register_shutdown_function (function () {
	/*
	Ne pas oublier de passer la directive display_errors à "stderr" dans php.ini, car sinon PHP aura déjà affiché le le message d'erreur avant d'appeler ce gestionnaire !
	*/
	$error = error_get_last ();
	if (!$error)
		return;
	$message = array ('file' => $error['file'], 'line' => $error['line'], 'message' => $error['message']);
	$response = new Response ("System", RESPONSE_ERROR_PHP, $message);
	echo ($response->toJSON ());
	exit ();
});

set_error_handler (function ($errorLevel, $errorMessage, $errorFile, $errorLine) {
	$message = array ('file' => $errorFile, 'line' => $errorLine, 'message' => $errorMessage);
	$response = new Response ("System", RESPONSE_ERROR_PHP, $message);
	echo ($response->toJSON ());
	exit ();
});

// Traiter la requête

try {
	$request = Request::fromJSON (getCGI ('request'));
	if (!class_exists ($request->service))
		throw (new Exception("Service \"$request->service\" does not exist."));
	$service = new $request->service;
	$service->run ($request->params);
}
catch (Exception $exception) {
	header ('Content-Type: application/json');
	$message = array ('file' => $exception->getFile (), 'line' => $exception->getLine (), 'message' => $exception->getMessage ());
	$response = new Response ("System", RESPONSE_ERROR_SERVICE, $message);
	echo ($response->toJSON ());
	exit ();
}
?>
